#!/bin/sh
#   Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Michael Tross <michael.tross@aizo.com>
#
#   This file is part of digitalSTROM Server.
#
#   mailconfig.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   mailconfig.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with netinfo.sh. If not, see <http://www.gnu.org/licenses/>.

echo "Content-Type: application/json"
echo ""

# source common functions
. ./errlib.sh
. ./jsonlib.sh

CONFIG_FILE="/etc/msmtprc"
DEFAULT_FILE="/etc/default/mail"

# load mail defaults
[ -f $DEFAULT_FILE ] && . $DEFAULT_FILE

# check msmtp config file
if [[ ! -f $CONFIG_FILE ]]; then
    json_error $ERR_MODE "Could not determine mail settings!"
    exit 1
fi

# ./mailconfig mode=... host=* from=* user=* password=* authOnOff=* sslOnOff=* tlsOnOff=* certOnOff=* admin=*
for PARAM in mode host from user password authOnOff sslOnOff tlsOnOff certOnOff admin; do
   P=$(echo "$@" | grep -o "${PARAM}=.*")
   eval $P
done

backup_settings() {
    DIFFER=1
    if [[ -f "$CONFIG_FILE.bkp" ]]; then
        cmp -s "$CONFIG_FILE" "$CONFIG_FILE.bkp"
        DIFFER="$?"
    fi

    if [[ "$DIFFER" -eq "1" ]]; then
        cp -f "$CONFIG_FILE" "$CONFIG_FILE.bkp" 1>&2
        if [[ "$?" -eq "0" ]]; then
            return $ERR_OK
        else
            return $ERR_BACKUP
        fi
    fi

    return $ERR_OK
}

restore_settings() {
    if [[ -f "$CONFIG_FILE.bkp" ]]; then
        cp -f "$CONFIG_FILE.bkp" "$CONFIG_FILE" 1>&2
        if [[ "$?" -eq "0" ]]; then
            return $ERR_OK
        fi
    fi
        
    return $ERR_RESTORE
}

config_load ()
{
   MAILHOST=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^host" | cut -d " " -f 2`
   MAILFROM=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^from" | cut -d " " -f 2`
   MAILUSER=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^user" | cut -d " " -f 2`
   MAILPASSWORD=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^password" | cut -d " " -f 2`
   MAILAUTH=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^auth" | cut -d " " -f 2`
   MAILSSL=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^tls [onf]" | cut -d " " -f 2`
   MAILTLS=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^tls_starttls" | cut -d " " -f 2`
   MAILCERT=`cat $CONFIG_FILE | sed -n '/^account/,$p' | grep "^tls_certcheck" | cut -d " " -f 2`
   [[ "$MAILTLS" == "on" ]] && MAILSSL="off"
}

# merge new settings
config_merge ()
{
   [[ -n "$host" ]] && MAILHOST=$host
   [[ -n "$from" ]] && MAILFROM=$from
   [[ -n "$user" ]] && MAILUSER=$user
   [[ -n "$password" ]] && MAILPASSWORD=$password
   [[ -n "$authOnOff" ]] && MAILAUTH=$authOnOff
   [[ -n "$sslOnOff" ]] && MAILSSL=$sslOnOff
   [[ -n "$tlsOnOff" ]] && MAILTLS=$tlsOnOff
   [[ -n "$certOnOff" ]] && MAILCERT=$certOnOff
   [[ -n "$admin" ]] && MAILADMIN=$admin
}

# rewrite msmtprc
config_save ()
{
   sed '/^#=== .* ===#/,$q' -i $CONFIG_FILE > /dev/null
   cat >> /etc/msmtprc <<EOT
host $MAILHOST
from $MAILFROM
auth $MAILAUTH
user $MAILUSER
password $MAILPASSWORD
EOT

    if [[ "$MAILSSL" == "on" ]]; then
        echo "tls on" >> $CONFIG_FILE
        echo "tls_starttls off" >> $CONFIG_FILE
    elif [[ "$MAILTLS" == "on" ]]; then
        echo "tls on" >> $CONFIG_FILE
        echo "tls_starttls on" >> $CONFIG_FILE
    else
        echo "tls off" >> $CONFIG_FILE
        echo "tls_starttls off" >> $CONFIG_FILE
    fi
   
    if [[ "$MAILCERT" == "off" ]]; then
        echo "tls_certcheck off" >> $CONFIG_FILE
        echo "tls_trust_file" >> $CONFIG_FILE
    else
        echo "tls_certcheck on" >> $CONFIG_FILE
    fi
    
    sed '/MAILADMIN/d' -i $DEFAULT_FILE >/dev/null
    echo "MAILADMIN=$MAILADMIN" >> $DEFAULT_FILE
}


if [[ "$mode" == "restore" ]]; then
    restore_settings
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not restore configuration!"
        exit 1
    fi
    json_ok
    
elif [[ "$mode" == "backup" ]]; then
    backup_settings
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not restore configuration!"
        exit 1
    fi
    json_ok

elif [[ "$mode" == "testconn" ]]; then
    config_load
    config_merge
    if [[ "$MAILCERT" == "off" ]]; then
        CERTARG="--tls-certcheck=off --tls-trust-file="
    else
        CERTARG="--tls-certcheck=on --tls-trust-file=/etc/ssl/certs/ca-certificates.crt"
    fi
    if [[ "$MAILSSL" == "on" ]]; then
        SSLARG="--tls=on --tls-starttls=off"
    elif [[ "$MAILTLS" == "on" ]]; then
        SSLARG="--tls=on --tls-starttls=on"
    else
        SSLARG="--tls=off"
    fi
    IFS='' RESPONSE=`msmtp --serverinfo --host=$MAILHOST --user=$MAILUSER --passwordeval="echo $MAILPASSWORD" $SSLARG $CERTARG 2>&1`
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
       json_error_info "$RETCODE" "Could not test mail server connection" "$RESPONSE"
       exit 1
    fi
    json_ok_info "$RESPONSE"

elif [[ "$mode" == "testmail" ]]; then
    config_load
    config_merge
    HEADER="From: dSS11 <$MAILFROM>\nTo: dSS11 Admin <$MAILFROM>\nSubject: dSS11 EMail Test\nDate: `date`\n\n";
    BODY="Server: `hostname`\n"
    if [[ "$MAILCERT" == "off" ]]; then
        CERTARG="--tls-certcheck=off --tls-trust-file="
    else
        CERTARG="--tls-certcheck=on --tls-trust-file=/etc/ssl/certs/ca-certificates.crt"
    fi
    if [[ "$MAILSSL" == "on" ]]; then
        SSLARG="--tls=on --tls-starttls=off"
    elif [[ "$MAILTLS" == "on" ]]; then
        SSLARG="--tls=on --tls-starttls=on"
    else
        SSLARG="--tls=off"
    fi
    IFS='' RESPONSE=`(echo -e $HEADER; echo -e $BODY) | msmtp -t --debug --host=$MAILHOST --from=$MAILFROM --auth=$MAILAUTH --user=$MAILUSER --passwordeval="echo $MAILPASSWORD" $SSLARG $CERTARG 2>&1`
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
       json_error_info "$RETCODE" "Could not send test mail to $MAILFROM" "$RESPONSE"
       exit 1
    fi
    json_ok_info "$RESPONSE"

elif [[ "$mode" == "mailset" ]]; then
    config_load
    config_merge
    config_save
    json_ok
    
else
    json_error $ERR_PARAM "Invalid option given!"
fi
